import {Component, Input, OnInit} from "@angular/core";
import {Layer, GanttChart} from "schedule";

@Component({
  selector: "demo-schedule-button-bar-layers-manager",
  templateUrl: "./demo-schedule-button-bar-layers-manager.component.html",
  styleUrls: ["./demo-schedule-button-bar-layers-manager.component.scss"]
})
export class DemoScheduleButtonBarLayersManagerComponent implements OnInit {

  // Dimensions

  layerRowHeightPx: number = 50;

  // Gantt

  @Input()
  gantt: GanttChart;

  layers: Layer[];
  layersOpacity: number[] = [];
  layerOrderTransition: boolean[] = [];

  ngOnInit(): void {
    this.layers = this.gantt.getLayers();
  }

  private _setLayerOrderTransition(indexArray: number[]): void {
    if (this.layerOrderTransition.length !== this.layers.length) {
      this.layerOrderTransition = this.layers.map(() => false);
    }
    indexArray.forEach(index => {
      this.layerOrderTransition[index] = true;
      setTimeout(() => this.layerOrderTransition[index] = false, 220);
    });
  }

  setLayerOpacity(index: number, inputEvent: InputEvent): void {
    if (this.layersOpacity.length !== this.layers.length) {
      this.layersOpacity = this.layers.map(layer => layer.getOpacity());
    }
    const value = parseFloat((inputEvent.target as HTMLInputElement).value);
    this.layers[index].setOpacity(value / 100);
    this.layersOpacity[index] = value / 100;
    this.gantt.redraw();
  }

  setLayerVisible(index: number, inputEvent: InputEvent): void {
    if (this.layersOpacity.length !== this.layers.length) {
      this.layersOpacity = this.layers.map(layer => layer.getOpacity());
    }
    const checked = (inputEvent.target as HTMLInputElement).checked;
    this.layers[index].setVisible(checked);
    this.layers[index].setOpacity(checked ? this.layersOpacity[index] || 1 : 0);
    this.gantt.redraw();
  }

  moveLayerToTop(from: number): void {
    this._setLayerOrderTransition(this.layers.map((_, i) => i).slice(0, from));
    const layer = this.layers[from];
    this.layers.splice(from, 1);
    this.layers.splice(0, 0, layer);
    this.gantt.redraw();
  }

  moveLayerToBottom(from: number): void {
    this._setLayerOrderTransition(this.layers.map((_, i) => i).slice(from, this.layers.length));
    const layer = this.layers[from];
    this.layers.splice(from, 1);
    this.layers.splice(this.layers.length, 0, layer);
    this.gantt.redraw();
  }

  moveLayer(from: number, to: number): void {
    this._setLayerOrderTransition([from, to]);
    const layer = this.layers[from];
    this.layers.splice(from, 1);
    this.layers.splice(to, 0, layer);
    this.gantt.redraw();
  }

  deleteLayer(index: number): void {
    if (this.layers[index].isDeletable()) {
      this.layers.splice(index, 1);
      this.layerOrderTransition.splice(index, 1);
      this.layersOpacity.splice(index, 1);
      this.gantt.redraw();
    }
    else {
      alert("This layer is protected from deletion");
    }
  }

  toggleAllLayers(visible: boolean): void {
    if (this.layersOpacity.length !== this.layers.length) {
      this.layersOpacity = this.layers.map(layer => layer.getOpacity());
    }
    this.layers.forEach((layer, i) => {
      layer.setOpacity(visible ? this.layersOpacity[i] || 1 : 0);
      layer.setVisible(visible);
    });
    this.gantt.redraw();
  }

}
