import {OnDestroy, Component, Input, TemplateRef} from "@angular/core";
import {GanttChart, ZoomMode, EventUtil} from "schedule";

import {demoScheduleButtonBarIcons} from "./models/demo-schedule-button-bar-icons.model";
import {DemoScheduleButtonBarLayersNameEnum} from "./models/demo-schedule-button-bar-layers-name-enum.model";
import {DemoScheduleButtonBarButtonTypesEnum} from "./models/demo-schedule-button-bar-button-types-enum.model";
import {enterLeaveAnimation} from "../app.animations";

@Component({
  selector: "demo-schedule-button-bar",
  templateUrl: "./demo-schedule-button-bar.component.html",
  styleUrls: ["./demo-schedule-button-bar.component.scss"],
  animations: [enterLeaveAnimation]
})
export class DemoScheduleButtonBarComponent implements OnDestroy {

  // Dimensions

  buttonHeightPx: number = 25;

  // Gantt

  @Input()
  gantt: GanttChart;

  // Buttons

  zoomModeOptions: DemoScheduleButtonBarSelectButtonOptions = {
    choices: [
      {icon: demoScheduleButtonBarIcons.center, label: "Keep Center", action: () => this.gantt.getTimeline().setZoomMode(ZoomMode.CENTER)},
      {icon: demoScheduleButtonBarIcons.earliest, label: "Keep Start", action: () => this.gantt.getTimeline().setZoomMode(ZoomMode.KEEP_START_TIME)},
      {icon: demoScheduleButtonBarIcons.latest, label: "Keep End", action: () => this.gantt.getTimeline().setZoomMode(ZoomMode.KEEP_END_TIME)}
    ]
  };

  gridOptions: DemoScheduleButtonBarSelectButtonOptions = {
    choices: [
      {icon: demoScheduleButtonBarIcons.gridOff, label: "Off", action: () => this.setGridMode("Off")},
      {icon: demoScheduleButtonBarIcons.grid, label: "1 level", action: () => this.setGridMode("1 level")},
      {icon: demoScheduleButtonBarIcons.grid, label: "2 levels", action: () => this.setGridMode("2 levels")}
    ]
  };

  layersViewData: DemoScheduleButtonBarModalButtonData = {
    position: {top: 0, left: 0}
  };

  demoScheduleButtonBarButtonTypesEnum: typeof DemoScheduleButtonBarButtonTypesEnum = DemoScheduleButtonBarButtonTypesEnum;
  lastButtonClicked: DemoScheduleButtonBarButton;
  activeModalButton: DemoScheduleButtonBarButton;
  modalDraggingState: DemoScheduleButtonBarDraggableModalState = {
    dragging: false,
    origin: {x: 0, y: 0},
    prevState: {x: 0, y: 0}
  };

  homeButton: DemoScheduleButtonBarButton = {label: "Home", icon: demoScheduleButtonBarIcons.home, buttonType: DemoScheduleButtonBarButtonTypesEnum.BASIC, action: () => window.location.href = "/"};
  nowButton: DemoScheduleButtonBarButton = {label: "Now", icon: demoScheduleButtonBarIcons.now, buttonType: DemoScheduleButtonBarButtonTypesEnum.BASIC, action: () => this.gantt.showNow()};
  earliestButton: DemoScheduleButtonBarButton = {label: "Earliest", icon: demoScheduleButtonBarIcons.earliest, buttonType: DemoScheduleButtonBarButtonTypesEnum.BASIC, action: () => this.gantt.showEarliestActivities()};
  latestButton: DemoScheduleButtonBarButton = {label: "Latest", icon: demoScheduleButtonBarIcons.latest, buttonType: DemoScheduleButtonBarButtonTypesEnum.BASIC, action: () => this.gantt.showLatestActivities()};
  allButton: DemoScheduleButtonBarButton = {label: "All", icon: demoScheduleButtonBarIcons.all, buttonType: DemoScheduleButtonBarButtonTypesEnum.BASIC, separator: true, action: () => this.gantt.showAllActivities()};
  zoomInButton: DemoScheduleButtonBarButton = {label: "Zoom In", icon: demoScheduleButtonBarIcons.zoomIn, buttonType: DemoScheduleButtonBarButtonTypesEnum.BASIC, action: () => this.gantt.zoomIn()};
  zoomOutButton: DemoScheduleButtonBarButton = {label: "Zoom Out", icon: demoScheduleButtonBarIcons.zoomOut, buttonType: DemoScheduleButtonBarButtonTypesEnum.BASIC, action: () => this.gantt.zoomOut()};
  zoomModeButton: DemoScheduleButtonBarButton = {label: "Zoom", icon: demoScheduleButtonBarIcons.zoomMode, buttonType: DemoScheduleButtonBarButtonTypesEnum.SELECT, buttonDataObject: this.zoomModeOptions, toggled: false, separator: true, action: () => this.toggleModalButtons(this.zoomModeButton)};
  layersButton: DemoScheduleButtonBarButton = {label: "Layers", icon: demoScheduleButtonBarIcons.layers, buttonType: DemoScheduleButtonBarButtonTypesEnum.MODAL, buttonDataObject: this.layersViewData, toggled: false, action: () => this.toggleModalButtons(this.layersButton)};
  radarButton: DemoScheduleButtonBarButton = {label: "Radar", icon: demoScheduleButtonBarIcons.radar, buttonType: DemoScheduleButtonBarButtonTypesEnum.MODAL, toggled: false, action: () => this.toggleModalButtons(this.radarButton)};
  gridModeButton: DemoScheduleButtonBarButton = {label: "Grid", icon: demoScheduleButtonBarIcons.grid, buttonType: DemoScheduleButtonBarButtonTypesEnum.SELECT, buttonDataObject: this.gridOptions, toggled: false, action: () => this.toggleModalButtons(this.gridModeButton)};
  calendarsButton: DemoScheduleButtonBarButton = {label: "Calendars", icon: demoScheduleButtonBarIcons.calendars, buttonType: DemoScheduleButtonBarButtonTypesEnum.BASIC, toggled: true, action: () => this.toggleCalendars()};
  nowLineButton: DemoScheduleButtonBarButton = {label: "Now Line", icon: demoScheduleButtonBarIcons.nowLine, buttonType: DemoScheduleButtonBarButtonTypesEnum.BASIC, toggled: true, action: () => this.toggleNowLine()};

  @Input()
  buttons: DemoScheduleButtonBarButton[] = [
    this.homeButton,
    this.nowButton,
    this.earliestButton,
    this.latestButton,
    this.allButton,
    this.zoomInButton,
    this.zoomOutButton,
    this.zoomModeButton,
    this.layersButton,
    this.radarButton,
    this.gridModeButton,
    this.calendarsButton,
    this.nowLineButton
  ];

  // Lifecycle

  ngOnDestroy(): void {
    window.removeEventListener("pointermove", this.draggingListener);
  }

  // Methods

  closeModal(button: DemoScheduleButtonBarButton): void {
    if (button?.toggled && (button?.buttonType === DemoScheduleButtonBarButtonTypesEnum.MODAL
      || button?.buttonType === DemoScheduleButtonBarButtonTypesEnum.SELECT)) {
      button.toggled = false;
      this.resetDraggingData(button);
    }
  }

  toggleModalButtons(button: DemoScheduleButtonBarButton): void {
    if (button.toggled) {
      if (this.activeModalButton && this.activeModalButton !== button) {
        const otherButton = this.activeModalButton;
        otherButton.toggled = false;
        this.resetDraggingData(otherButton);
      }
      this.activeModalButton = button;
    }
    if (!button.toggled) {
      this.resetDraggingData(button);
    }
  }

  buttonAction(button: DemoScheduleButtonBarButton): void {
    this.lastButtonClicked = button;
    if (button.toggled !== undefined) {
      button.toggled = !button.toggled;
    }
    if (button.action !== undefined) {
      button.action();
    }
  }

  displayLabel(button: DemoScheduleButtonBarButton | DemoScheduleButtonBarSelectButtonChoice): string {
    return typeof button.label === "function" ? button.label() : button.label;
  }

  setSelection(button: DemoScheduleButtonBarButton, choice: DemoScheduleButtonBarSelectButtonChoice): void {
    if (button.toggled) {
      button.toggled = false;
      button.label = choice.label;
      button.icon = choice.icon;
      choice.action();
    }
  }

  setGridMode(mode: string): void {
    const backgroundSystemLayers = this.gantt.getGraphics().getBackgroundSystemLayers();
    const innerLinesLayer = backgroundSystemLayers.find(it => it.getName() === DemoScheduleButtonBarLayersNameEnum.INNER_LINES_LAYER_NAME);
    const gridLinesLayer = backgroundSystemLayers.find(it => it.getName() === DemoScheduleButtonBarLayersNameEnum.GRID_LINES_LAYER_NAME);
    if (mode === "Off") {
      innerLinesLayer.setVisible(false);
      gridLinesLayer.setVisible(false);
    } else if (mode === "1 level") {
      innerLinesLayer.setVisible(true);
      gridLinesLayer.setVisible(false);
    } else if (mode === "2 levels") {
      innerLinesLayer.setVisible(true);
      gridLinesLayer.setVisible(true);
    }
    this.gantt.redraw();
  }

  toggleCalendars(): void {
    const calendarLayer = this.gantt.getGraphics().getBackgroundSystemLayers().find(it => it.getName() === DemoScheduleButtonBarLayersNameEnum.CALENDAR_LINES_LAYER_NAME);
    calendarLayer.setVisible(this.calendarsButton.toggled);
    this.gantt.redraw();
  }

  toggleNowLine(): void {
    const nowLineAboveLayer = this.gantt.getGraphics().getAboveViewTransversalSystemLayers().find(it => it.getName() === DemoScheduleButtonBarLayersNameEnum.NOW_LINE_ABOVE_LAYER);
    nowLineAboveLayer.setVisible(this.nowLineButton.toggled);
    this.gantt.redraw();
  }

  enableModalDragMode(state: boolean, button: DemoScheduleButtonBarButton, pointerEvent: PointerEvent): void {
    const pointerEventTarget = EventUtil.getTarget(pointerEvent);
    if (pointerEventTarget instanceof HTMLButtonElement || pointerEventTarget instanceof HTMLInputElement) {
      return;
    }
    if (state) {
      window.addEventListener("pointermove", this.draggingListener);
      const position = (button.buttonDataObject as DemoScheduleButtonBarModalButtonData)?.position;
      if (position) {
        this.modalDraggingState.dragging = true;
        this.modalDraggingState.origin.x = pointerEvent.clientX;
        this.modalDraggingState.origin.y = pointerEvent.clientY;
        this.modalDraggingState.prevState.x = position.left;
        this.modalDraggingState.prevState.y = position.top;
      }
    }
    else {
      this.modalDraggingState.dragging = false;
      window.removeEventListener("pointermove", this.draggingListener);
    }
  }

  draggingListener: (pointerEvent: PointerEvent) => void = (pointerEvent) => {
    if (this.modalDraggingState.dragging) {
      const position = (this.lastButtonClicked.buttonDataObject as DemoScheduleButtonBarModalButtonData)?.position;
      if (position) {
        position.left = pointerEvent.clientX - this.modalDraggingState.origin.x + this.modalDraggingState.prevState.x;
        position.top = pointerEvent.clientY - this.modalDraggingState.origin.y + this.modalDraggingState.prevState.y;
      }
    }
  }

  resetDraggingData(button: DemoScheduleButtonBarButton): void {
    const position = (button.buttonDataObject as DemoScheduleButtonBarModalButtonData)?.position;
    if (position) {
      setTimeout(() => {
        position.top = 0;
        position.left = 0;
        this.modalDraggingState.dragging = false;
        this.modalDraggingState.prevState.x = 0;
        this.modalDraggingState.prevState.y = 0;
      }, 200);
    }
  }

}

export interface DemoScheduleButtonBarButton {
  label?: string | (() => string);
  icon?: string;
  buttonType?: DemoScheduleButtonBarButtonTypesEnum;
  buttonDataObject?: DemoScheduleButtonBarSelectButtonOptions | DemoScheduleButtonBarModalButtonData;
  toggled?: boolean;
  hidden?: boolean;
  disabled?: boolean;
  separator?: boolean;
  action?: () => void;
}

export interface DemoScheduleButtonBarSelectButtonOptions {
  choices?: DemoScheduleButtonBarSelectButtonChoice[];
}

export interface DemoScheduleButtonBarSelectButtonChoice {
  icon?: string;
  label?: string | (() => string);
  action?: () => void;
}

export interface DemoScheduleButtonBarModalButtonData {
  templateRef?: TemplateRef<any>;
  position?: Position;
}

export interface DemoScheduleButtonBarDraggableModalState {
  dragging?: boolean;
  origin?: Point;
  prevState: Point;
}

export interface Position {
  top?: number;
  left?: number;
}

export interface Point {
  x?: number;
  y?: number;
}
