import {ActivityBarRenderer, ActivityBarRendererTextPosition, ActivityBounds, ActivityRef, Color, ViewPosition} from "schedule";

import {Flight, Aircraft} from "./aircraft.model";

export class FlightRenderer extends ActivityBarRenderer<Flight, Aircraft> {

  constructor(graphics) {
    super(graphics, "Flight activity renderer");
  }

  protected drawActivity(activityRef: ActivityRef<Flight>, position: ViewPosition, ctx: CanvasRenderingContext2D,x: number, y: number, w: number, h: number, selected: boolean, hover: boolean, highlighted: boolean, pressed: boolean): ActivityBounds {
    const layerName = activityRef.getLayer().getName();

    this.setBarHeight(h * 0.75);
    this.setCornersRounded(true);
    this.setStroke(Color.BLACK);

    if (layerName === "Cargo") {
      this.setFill(Color.CORNFLOWERBLUE);
      this.setTextFill(Color.WHITE);
    }
    else if (layerName === "Training") {
      this.setFill(Color.ALICEBLUE);
      this.setTextFill(Color.BLACK);
    }
    else if (layerName === "Charter") {
      this.setFill(Color.ANTIQUEWHITE);
      this.setTextFill(Color.BLACK);
    }

    const bounds = super.drawActivity(activityRef, position, ctx, x, y, w, h, selected, hover, highlighted, pressed);
    
    ctx.font = "italic 11px Verdana";
    this.drawText(activityRef, layerName, ActivityBarRendererTextPosition.CENTER, ctx, x, y, w, h, selected, hover, highlighted, pressed);

    return bounds;
  }

}
