import {Component, Injector} from "@angular/core";
import {AppTransactionalService, ChronoUnit, DefaultScheduleGanttComponentBase, GanttChart, GanttLayout, Layer} from "schedule";

import {Aircraft, Flight, FlightData} from "./model/aircraft.model";
import {FlightRenderer} from "./model/flight-renderer.model";


@Component({
  selector: "app-simple-aircraft-gantt",
  templateUrl: "./simple-aircraft-gantt.component.html",
  styleUrls: ["./simple-aircraft-gantt.component.scss"]
})
export class SimpleAircraftGanttComponent extends DefaultScheduleGanttComponentBase<Aircraft> {

  // Create the Gantt chart.
  gantt = new GanttChart<Aircraft>(
    undefined,        // Model Object.
    this._injector    // Provided by DefaultScheduleGanttComponentBase.
  );

  // Required by DefaultScheduleGanttComponentBase.
  constructor(_appTransactionalService: AppTransactionalService, _injector: Injector) {
    super(_appTransactionalService, _injector);
  }

  start(): void {

    // Register the flight renderer
    const graphics = this.gantt.getGraphics();
    graphics.setActivityRenderer(
      Flight,
      GanttLayout,
      new FlightRenderer(graphics)
    );

    // Create the Layers based on service type.
    const cargoLayer = new Layer("Cargo");
    const trainingLayer = new Layer("Training");
    const charterLayer = new Layer("Charter");
    this.gantt.getLayers().push(cargoLayer, trainingLayer, charterLayer);

    // Create the Flights.
    const flight1 = new Flight(new FlightData("Flight 1", 0));
    const flight2 = new Flight(new FlightData("Flight 2", 9));
    const flight3 = new Flight(new FlightData("Flight 3", 18));
    const flight4 = new Flight(new FlightData("Flight 4", 0));
    const flight5 = new Flight(new FlightData("Flight 5", 9));
    const flight6 = new Flight(new FlightData("Flight 6", 18));

    // Create an Aircraft and add its Activities on the corresponding Layer.
    const a380 = new Aircraft("A 380");
    a380.addActivity(cargoLayer, flight1);
    a380.addActivity(trainingLayer, flight2);
    a380.addActivity(charterLayer, flight3);

    const b747 = new Aircraft("B 747");
    b747.addActivity(cargoLayer, flight4);
    b747.addActivity(trainingLayer, flight5);
    b747.addActivity(charterLayer, flight6);

    // Add the aircraft rows to the Gantt Chart.
    const rows = [a380, b747];
    this.gantt.addRows(...rows);

    // Timeline initial scale.
    this.gantt.getTimeline().showTemporalUnit(ChronoUnit.DAYS, 400);
  }

}
